program main

  use vtype_module

  use fi_module

  use ni3_module

  use ac_merge, only : &
    & ACMergeWaveNumber, &
    & ACMergeVerticalDomain

#ifdef MPI
  use mpi
#endif

  implicit none


  character(extstr) :: CtlFN3 = 'many.conf'
  character(extstr) :: CtlFN4 = 'mpisubloop.conf'

  integer                   :: iostat
  integer(i4b)              :: FileUnit


  integer            :: NProcsMPISubLoop

  integer :: iTemp
  integer :: iTemps
  integer :: iTempe
  integer :: Mol1Num
  integer :: iVMRMol1
  integer :: iVMRMol1s
  integer :: iVMRMol1e
  integer :: Mol2Num
  integer :: iVMRMol2
  integer :: iVMRMol2s
  integer :: iVMRMol2e
  integer :: Mol3Num
  integer :: iVMRMol3
  integer :: iVMRMol3s
  integer :: iVMRMol3e

  integer :: NTemp
  integer :: NVMRMol1
  integer :: NVMRMol2
  integer :: NVMRMol3

  integer :: iLoop
  integer :: NLoop
  integer :: iLoops
  integer :: iLoope

  integer :: iRank

  integer :: IErr
  integer :: NProcs
  integer :: MyRank

  character(len=extstr)     :: FileNameTail
  character(len=extstr)     :: InputFileName0
  character(len=extstr)     :: OutputFileName


  namelist /mpisubloop_nml/ &
    & NProcsMPISubLoop

  namelist /many_nml/ &
    & iTemps, iTempe, &
    & Mol1Num, iVMRMol1s, iVMRMol1e, &
    & Mol2Num, iVMRMol2s, iVMRMol2e, &
    & Mol3Num, iVMRMol3s, iVMRMol3e


#ifdef MPI
  call mpi_init( IErr )
  call mpi_comm_size( mpi_comm_world, NProcs, IErr )
  call mpi_comm_rank( mpi_comm_world, MyRank, IErr )
#else
  NProcs = 1
  MyRank = 0
#endif


  NProcsMPISubLoop = 0


  ! Read a namelist file

  call fi_open( CtlFN3, "read", FileUnit )
  !
  rewind( FileUnit )
  read( FileUnit, nml = many_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist in many_nml is something wrong.'
  end if
  write( 6, nml = many_nml )
  !
  close( FileUnit )

  call fi_open( CtlFN4, "read", FileUnit )
  !
  rewind( FileUnit )
  read( FileUnit, nml = mpisubloop_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist in mpisubloop_nml is something wrong.'
  end if
  write( 6, nml = mpisubloop_nml )
  !
  close( FileUnit )


  NTemp    = iTempe    - iTemps    + 1
  NVMRMol1 = iVMRMol1e - iVMRMol1s + 1
  NVMRMol2 = iVMRMol2e - iVMRMol2s + 1
  NVMRMol3 = iVMRMol3e - iVMRMol3s + 1

  NLoop = NTemp * NVMRMol1 * NVMRMol2 * NVMRMol3

  iLoope = 0
  do iRank = 0, MyRank
    iLoops = iLoope + 1
    iLoope = iLoops + NLoop / NProcs - 1
    if ( iRank < mod( NLoop, NProcs ) ) then
      iLoope = iLoope + 1
    end if
  end do
  ! check
  if ( iLoope > NLoop ) then
    write( 6, * ) 'Unexpected number of maximum of loop: ', iLoope, NLoop
  end if


  write( 6, * ) MyRank, iLoops, iLoope

  do iLoop = iLoops, iLoope

    iTemp       = iTemps       &
      & + ( iLoop - 1 ) / ( NVMRMol1 * NVMRMol2 * NVMRMol3 )
    iVMRMol1 = iVMRMol1s &
      & + mod( ( iLoop - 1 ) / ( NVMRMol2 * NVMRMol3 ), NVMRMol1 )
    iVMRMol2 = iVMRMol2s &
      & + mod( ( iLoop - 1 ) / ( NVMRMol3 ), NVMRMol2 )
    iVMRMol3 = iVMRMol3s &
      & + mod( ( iLoop - 1 ), NVMRMol3 )

    write( FileNameTail, '(a,i3.3,3(a,i2.2,a,i3.3),a)' ) &
      & "T-", iTemp, &
      & "_M", Mol1Num, "-", iVMRMol1, &
      & "_M", Mol2Num, "-", iVMRMol2, &
      & "_M", Mol3Num, "-", iVMRMol3, &
      & ".nc"
    InputFileName0 = "out/ac_" // trim( FileNameTail )
    OutputFileName = InputFileName0

    write( 6, * ) 'Output file : ', trim( OutputFileName )

#ifdef PARALLEL_WAVENUMBER
    write( 6, * ) "Merge wavenumbers"
    call ACMergeWaveNumber( &
      & NProcsMPISubLoop, InputFileName0, OutputFileName &
      & )
#else
    write( 6, * ) "Merge vertical domain"
    call ACMergeVerticalDomain( &
      & NProcsMPISubLoop, InputFileName0, OutputFileName &
      & )
#endif

  end do


#ifdef MPI
  call mpi_finalize( IErr )
#endif


  write( 6, * ) 'Finish'


end program main
