program main

  use vtype_module
  use fi_module
  use ni3_module
  use netcdf

  implicit none

  integer                        :: NcID
  character(extstr)              :: Mode
  character(extstr)              :: Name
  character(extstr)              :: StdName
  character(extstr)              :: LongName
  character(extstr)              :: Units
  integer                        :: NDims
  character(extstr), allocatable :: a_DimNames(:)

  character(extstr)              :: OutDir

  integer                :: NPress
  integer                :: NTemp
  real(8), allocatable   :: r_Press(:)
  real(8), allocatable   :: t_Temp (:)
  logical, allocatable   :: rt_FlagCalc(:,:)

  real(8), allocatable   :: r_Temp    (:)
  integer, allocatable   :: r_FlagCalc(:)

  integer                :: NMol
  integer, allocatable   :: m_MolNum(:)

  real(8), allocatable   :: rm_VMR(:,:)

  real(8)                :: Ps
  real(8)                :: Ts

  real(8) :: PressMin
  real(8) :: PressMax

  real(8) :: TempMin
  real(8) :: TempMax

  real(8) :: Press
  real(8) :: Temp

  character(128) :: NcFN
  character(128) :: ctlfn = 'mkprofile.conf'

  integer :: FileUnit
  integer :: iostat

  integer :: iTemp
  integer :: iVMRKeyMol1
  integer :: iVMRKeyMol2
  integer :: iVMRKeyMol3

  integer :: KeyMol1Num
  integer :: KeyMol2Num
  integer :: KeyMol3Num
  integer :: NVMRKeyMol1
  integer :: NVMRKeyMol2
  integer :: NVMRKeyMol3
  real(8), allocatable :: u_VMRsKeyMol1(:)
  real(8), allocatable :: v_VMRsKeyMol2(:)
  real(8), allocatable :: w_VMRsKeyMol3(:)

  integer :: IndexKeyMol1
  integer :: IndexKeyMol2
  integer :: IndexKeyMol3

  logical :: FlagPressZero = .true.
!!$  logical :: FlagPressZero = .false.

  character(extstr)              :: NcFNVMR
  logical          , allocatable :: a_FlagVMRSetFromFile(:)

  logical                        :: FlagReadVMRFile
  integer                        :: NMolNumVMRFile
  integer                        :: NPressVMRFile
  integer          , allocatable :: m_MolNumVMRFile(:)
  real(dp)         , allocatable :: r_PressVMRFile (:)
  real(dp)         , allocatable :: rm_VMRFile     (:,:)

  integer, parameter :: MaxNmlArySize = 256
  integer            :: MolNum(MaxNmlArySize)
  real(8)            :: VMR(MaxNmlArySize)
  real(8)            :: VMRsKeyMol1(MaxNmlArySize)
  real(8)            :: VMRsKeyMol2(MaxNmlArySize)
  real(8)            :: VMRsKeyMol3(MaxNmlArySize)
  logical            :: FlagVMRSetFromFile(MaxNmlArySize)

  integer :: k
  integer :: m
  integer :: m2


  namelist /output_nml/ &
    & OutDir

  namelist /molecule_nml/ &
    & MolNum, VMR, &
    & NcFNVMR, FlagVMRSetFromFile

  namelist /range_nml/ &
    & PressMax, PressMin, NPress, &
    & TempMin, TempMax, NTemp, &
    & KeyMol1Num, NVMRKeyMol1, VMRsKeyMol1, &
    & KeyMol2Num, NVMRKeyMol2, VMRsKeyMol2, &
    & KeyMol3Num, NVMRKeyMol3, VMRsKeyMol3


  MolNum             = -1
  MolNum(1)          =  1
  VMR                = -1.0d0
  VMR(1)             = 1.0d-2
!!$  VMR(2)             = 369.0d-6
!!$  VMR(3)             = 1.0d-6
  !
  NcFNVMR               = ""
  FlagVMRSetFromFile(:) = .false.


  PressMax      = 1.0d5
  PressMin      = 1.0d2
  NPress        = 31

  TempMin = 150.0d0
  TempMax = 350.0d0
  NTemp   = 7


  KeyMol1Num         = 1
  NVMRKeyMol1        = 1
  VMRsKeyMol1(:)     = -1.0d0
  VMRsKeyMol1(1)     =  0.0d0
  !
  KeyMol2Num         = 2
  NVMRKeyMol2        = 1
  VMRsKeyMol2(:)     = -1.0d0
  VMRsKeyMol2(1)     =  0.0d0
  !
  KeyMol3Num         = 3
  NVMRKeyMol3        = 1
  VMRsKeyMol3(:)     = -1.0d0
  VMRsKeyMol3(1)     =  0.0d0

  OutDir = "data_profile"

  call fi_open( CtlFN, "read", FileUnit )
  !
  rewind( FileUnit )
  read( FileUnit, nml = molecule_nml, iostat = iostat )
  if ( iostat /= 0 ) then
    stop 'Namelist, molecule_nml, is something wrong.'
  end if
  write( 6, nml = molecule_nml )
  !
  rewind( FileUnit )
  read( FileUnit, nml = range_nml, iostat = iostat )
  if ( iostat /= 0 ) then
    stop 'Namelist, range_nml, is something wrong.'
  end if
  write( 6, nml = range_nml )
  !
  rewind( FileUnit )
  read( FileUnit, nml = output_nml, iostat = iostat )
  if ( iostat > 0 ) then
    stop 'Namelist, output_nml, is something wrong.'
  end if
  write( 6, nml = output_nml )
  close( FileUnit )


  if ( FlagPressZero      ) NPress      = NPress      + 1

  allocate( u_VMRsKeyMol1( NVMRKeyMol1 ) )
  u_VMRsKeyMol1 = VMRsKeyMol1(1:NVMRKeyMol1)
  allocate( v_VMRsKeyMol2( NVMRKeyMol2 ) )
  v_VMRsKeyMol2 = VMRsKeyMol2(1:NVMRKeyMol2)
  allocate( w_VMRsKeyMol3( NVMRKeyMol3 ) )
  w_VMRsKeyMol3 = VMRsKeyMol3(1:NVMRKeyMol3)

  ! check
  do iVMRKeyMol1 = 1+1, NVMRKeyMol1
    if ( u_VMRsKeyMol1(iVMRKeyMol1-1) >= u_VMRsKeyMol1(iVMRKeyMol1) ) then
      write( 6, * ) 'Error : KeyMol1, order of vmr is inappropriate.'
      write( 6, * ) u_VMRsKeyMol1
      stop
    end if
  end do
  do iVMRKeyMol2 = 1+1, NVMRKeyMol2
    if ( v_VMRsKeyMol2(iVMRKeyMol2-1) >= v_VMRsKeyMol2(iVMRKeyMol2) ) then
      write( 6, * ) 'Error : KeyMol2, order of vmr is inappropriate.'
      write( 6, * ) v_VMRsKeyMol2
      stop
    end if
  end do
  do iVMRKeyMol3 = 1+1, NVMRKeyMol3
    if ( w_VMRsKeyMol3(iVMRKeyMol3-1) >= w_VMRsKeyMol3(iVMRKeyMol3) ) then
      write( 6, * ) 'Error : KeyMol3, order of vmr is inappropriate.'
      write( 6, * ) w_VMRsKeyMol3
      stop
    end if
  end do


  do m = 1, MaxNmlArySize
    if ( MolNum(m) <= 0.0d0 ) exit
  end do
  NMol = m - 1
  do m = 1, NMol
    if ( VMR(m) < 0.0d0 ) then
      write( 6, * ) 'VMR(',m,') is negative.'
      stop
    end if
  end do
  !
  allocate( m_MolNum(NMol) )
  allocate( rm_VMR(NPress, NMol) )
  allocate( a_FlagVMRSetFromFile(NMol) )

  m_MolNum = MolNum(1:NMol)
  do m = 1, NMol
    rm_VMR(:,m) = VMR(m)
  end do
  !
  a_FlagVMRSetFromFile = FlagVMRSetFromFile(1:NMol)

  do m = 1, NMol
    if ( m_MolNum(m) == KeyMol1Num ) exit
  end do
  if ( m > NMol ) then
    write( 6, * ) 'Unable to find KeyMol1=', KeyMol1Num, ' in MolNum.'
    write( 6, * ) '  MolNum = ', m_MolNum
    stop
  end if
  if ( a_FlagVMRSetFromFile(m) ) then
    if ( NVMRKeyMol1 /= 1 ) then
      write( 6, * ) 'For key molecule 1:'
      write( 6, * ) '  NVMRKeyMolX has to be 1, when FlagVMRSetFromFile is .true.'
      stop
    end if
  end if
  IndexKeyMol1 = m
  !
  do m = 1, NMol
    if ( m_MolNum(m) == KeyMol2Num ) exit
  end do
  if ( m > NMol ) then
    write( 6, * ) 'Unable to find KeyMol2=', KeyMol1Num, ' in MolNum.'
    write( 6, * ) '  MolNum = ', m_MolNum
    stop
  end if
  if ( a_FlagVMRSetFromFile(m) ) then
    if ( NVMRKeyMol2 /= 1 ) then
      write( 6, * ) 'For key molecule 2:'
      write( 6, * ) '  NVMRKeyMolX has to be 1, when FlagVMRSetFromFile is .true.'
      stop
    end if
  end if
  IndexKeyMol2 = m
  !
  do m = 1, NMol
    if ( m_MolNum(m) == KeyMol3Num ) exit
  end do
  if ( m > NMol ) then
    write( 6, * ) 'Unable to find KeyMol3=', KeyMol1Num, ' in MolNum.'
    write( 6, * ) '  MolNum = ', m_MolNum
    stop
  end if
  if ( a_FlagVMRSetFromFile(m) ) then
    if ( NVMRKeyMol3 /= 1 ) then
      write( 6, * ) 'For key molecule 3:'
      write( 6, * ) '  NVMRKeyMolX has to be 1, when FlagVMRSetFromFile is .true.'
      stop
    end if
  end if
  IndexKeyMol3 = m


  allocate( r_Press    (NPress) )
  allocate( t_Temp     (NTemp ) )
  allocate( rt_FlagCalc(NPress, NTemp) )

  do k = 1, NPress
!!$    Press = 10.0d0**( log10( PressMax ) - dble( k - 1 ) / dble(NPressPer10) )
    if ( FlagPressZero ) then
      if ( k < NPress ) then
        Press = &
          &   log( PressMin / PressMax ) / dble(NPress-1-1) * dble( k-1 ) &
          & + log( PressMax )
        Press = exp( Press )
      else
        Press = 0.0d0
      end if
    else
      Press = &
        &   log( PressMin / PressMax ) / dble(NPress-1  ) * dble( k-1 ) &
        & + log( PressMax )
      Press = exp( Press )
    end if

    r_Press(k) = Press
  end do
  do iTemp = 1, NTemp
    Temp = ( TempMax - TempMin ) / ( NTemp - 1 ) * ( iTemp - 1 ) + TempMin
    t_Temp(iTemp) = Temp
  end do


  ! Set flag
  call SetFlag( &
    & NPress, NTemp,   & ! (in)
    & r_Press, t_Temp, & ! (in)
    & rt_FlagCalc      & ! (out)
    & )


!!$  do m = 1+1, NMol
!!$    do k = 1, NPress
!!$      if ( m == 2 ) then
!!$        rm_VMR(k,m) = VMRCO2
!!$      end if
!!$      if ( m == 3 ) then
!!$        rm_VMR(k,m) = VMRO3
!!$      end if
!!$      if ( m > 3 ) then
!!$        rm_VMR(k,m) = 0.0d0
!!$      end if
!!$    end do
!!$  end do



  FlagReadVMRFile = .false.
  do m = 1, NMol
    if ( a_FlagVMRSetFromFile(m) ) then
      FlagReadVMRFile = .true.
    end if
  end do
  if ( FlagReadVMRFile ) then
    Mode = "read"
    call ni3_open( NcFNVMR, Mode, NcID )
    Name = "MolNum"
    call ni3_inq_dimlen( NcID, Name, NMolNumVMRFile )
    Name = "Press"
    call ni3_inq_dimlen( NcID, Name, NPressVMRFile )
    !
    allocate( m_MolNumVMRFile( NMolNumVMRFile ) )
    allocate( r_PressVMRFile ( NPressVMRFile ) )
    allocate( rm_VMRFile     ( NPressVMRFile, NMolNumVMRFile ) )
    !
    Name = "MolNum"
    call ni3_get_var( NcID, Name, m_MolNumVMRFile )
    Name = "Press"
    call ni3_get_var( NcID, Name, r_PressVMRFile )
    Name = "VMR"
    call ni3_get_var( NcID, Name, rm_VMRFile )
    !
    call ni3_close( NcID )

    do m = 1, NMol
      if ( a_FlagVMRSetFromFile(m) ) then

        do m2 = 1, NMolNumVMRFile
          if ( m_MolNum(m) == m_MolNumVMRFile(m2) ) then
            exit
          end if
        end do
        if ( m2 > NMolNumVMRFile ) then
          write( 6, * ) 'Unable to find molecular number ', m, ' in file.'
          stop
        end if
        call Interpolate1D( &
          & NPressVMRFile, r_PressVMRFile, rm_VMRFile(:,m2), &
          & NPress, r_Press,                                 &
          & rm_VMR(:,m)                                      &
          & )

      end if
    end do

    deallocate( m_MolNumVMRFile )
    deallocate( r_PressVMRFile  )
    deallocate( rm_VMRFile      )
  end if


  allocate( r_Temp    (NPress) )
  allocate( r_FlagCalc(NPress) )

  do iVMRKeyMol3 = 1, NVMRKeyMol3
    do iVMRKeyMol2 = 1, NVMRKeyMol2
      do iVMRKeyMol1 = 1, NVMRKeyMol1

        ! mixing ratio of key mol. 1
        if ( a_FlagVMRSetFromFile(IndexKeyMol1) ) then
          ! VMR has already been set.
        else
          rm_VMR(:,IndexKeyMol1) = u_VMRsKeyMol1(iVMRKeyMol1)
        end if

        ! mixing ratio of key mol. 2
        if ( a_FlagVMRSetFromFile(IndexKeyMol2) ) then
          ! VMR has already been set.
        else
          rm_VMR(:,IndexKeyMol2) = v_VMRsKeyMol2(iVMRKeyMol2)
        end if


        ! mixing ratio of key mol. 3
        if ( a_FlagVMRSetFromFile(IndexKeyMol3) ) then
          ! VMR has already been set.
        else
          rm_VMR(:,IndexKeyMol3) = w_VMRsKeyMol3(iVMRKeyMol3)
        end if

        do iTemp = 1, NTemp
          r_Temp     = t_Temp(iTemp)

          Ps = r_Press(1)
          Ts = r_Temp(1)

          do k = 1, NPress
            if ( rt_FlagCalc(k,iTemp) ) then
              r_FlagCalc(k) = 1
            else
              r_FlagCalc(k) = 0
            end if
          end do


          write( NcFN, '(a,i3.3,3(a,i2.2,a,i3.3),a)' ) &
            & trim( OutDir ) // '/Profile_T-', iTemp, &
            & '_M', KeyMol1Num, '-', iVMRKeyMol1, &
            & '_M', KeyMol2Num, '-', iVMRKeyMol2, &
            & '_M', KeyMol3Num, '-', iVMRKeyMol3, &
            & '.nc'


          ! Output

          Mode = "new"
          call ni3_open( NcFN, Mode, NcID )

          Name     = "Press"
          StdName  = Name
          LongName = "Pressure"
          Units    = "Pa"

          call ni3_set_dim( NcID, Name, NF90_DOUBLE, r_Press, &
            & StdName, LongName, Units )
          Name     = "MolNum"
          StdName  = Name
          LongName = "Molecular number"
          Units    = "1"
          call ni3_set_dim( NcID, Name, NF90_DOUBLE, m_MolNum, &
            & StdName, LongName, Units )


          NDims = 1
          allocate( a_DimNames( NDims ) )
          !
          Name     = "Temp"
          StdName  = Name
          LongName = "Temperature"
          Units    = "K"
          a_DimNames( 1 ) = "Press"
          call ni3_def_var( NcID, Name, NF90_DOUBLE, NDims, a_DimNames, &
            & StdName, LongName, Units )
          call ni3_put_var( NcID, Name, r_Temp )
          !
          deallocate( a_DimNames )


          NDims = 2
          allocate( a_DimNames( NDims ) )
          !
          Name     = "VMR"
          StdName  = Name
          LongName = "Volume mixing ratio"
          Units    = "1"
          a_DimNames( 1 ) = "Press"
          a_DimNames( 2 ) = "MolNum"
          call ni3_def_var( NcID, Name, NF90_DOUBLE, NDims, a_DimNames, &
            & StdName, LongName, Units )
          call ni3_put_var( NcID, Name, rm_VMR )
          !
          deallocate( a_DimNames )

          !-------------------

          NDims = 0
          allocate( a_DimNames( NDims ) )
          !
          Name     = "Ts"
          StdName  = Name
          LongName = "Surface temperature"
          Units    = "K"
          call ni3_def_var( NcID, Name, NF90_DOUBLE, NDims, a_DimNames, &
            & StdName, LongName, Units )
          call ni3_put_var( NcID, Name, Ts )
          !
          Name     = "Ps"
          StdName  = Name
          LongName = "Surface pressure"
          Units    = "Pa"
          call ni3_def_var( NcID, Name, NF90_DOUBLE, NDims, a_DimNames, &
            & StdName, LongName, Units )
          call ni3_put_var( NcID, Name, Ps )
          !
          deallocate( a_DimNames )

          NDims = 1
          allocate( a_DimNames( NDims ) )
          !
          Name     = "FlagCalc"
          StdName  = Name
          LongName = "flag for calculation"
          Units    = "1"
          a_DimNames( 1 ) = "Press"
          call ni3_def_var( NcID, Name, NF90_INT, NDims, a_DimNames, &
            & StdName, LongName, Units )
          call ni3_put_var( NcID, Name, r_FlagCalc )
          !
          deallocate( a_DimNames )

          call ni3_close( NcID )

        end do
      end do
    end do
  end do


  ! Output netcdf file for variable range
  !
  NcFN = trim( OutDir ) // '/VariableRange.nc'

  Mode = "new"
  call ni3_open( NcFN, Mode, NcID )

  Name     = "Press"
  StdName  = Name
  LongName = "Pressure"
  Units    = "Pa"
  call ni3_set_dim( NcID, Name, NF90_DOUBLE, r_Press, &
    & StdName, LongName, Units )

  Name     = "MolNum"
  StdName  = Name
  LongName = "Molecular number"
  Units    = "1"
  call ni3_set_dim( NcID, Name, NF90_DOUBLE, m_MolNum, &
    & StdName, LongName, Units )
  Name     = "Temp"
  StdName  = Name
  LongName = "Temperature"
  Units    = "K"
  call ni3_set_dim( NcID, Name, NF90_DOUBLE, t_Temp, &
    & StdName, LongName, Units )
  Name     = "VMR1"
  StdName  = Name
  LongName = "Volume mixing ratio of key molecular species 1"
  Units    = "1"
  call ni3_set_dim( NcID, Name, NF90_DOUBLE, u_VMRsKeyMol1, &
    & StdName, LongName, Units )
  call ni3_put_att( NcID, Name, "MolNum", KeyMol1Num )
  Name     = "VMR2"
  StdName  = Name
  LongName = "Volume mixing ratio of key molecular species 2"
  Units    = "1"
  call ni3_set_dim( NcID, Name, NF90_DOUBLE, v_VMRsKeyMol2, &
    & StdName, LongName, Units )
  call ni3_put_att( NcID, Name, "MolNum", KeyMol2Num )
  Name     = "VMR3"
  StdName  = Name
  LongName = "Volume mixing ratio of key molecular species 3"
  Units    = "1"
  call ni3_set_dim( NcID, Name, NF90_DOUBLE, w_VMRsKeyMol3, &
    & StdName, LongName, Units )
  call ni3_put_att( NcID, Name, "MolNum", KeyMol3Num )

  call ni3_close( NcID )


  stop

  !----------------------------------------------------------------------------

contains

  !----------------------------------------------------------------------------

  subroutine Interpolate1D( &
    & NLev, a_Press, a_Array,         &
    & kmax, z_Press,                  &
    & z_Array                         &
    & )

    integer , intent(in ) :: NLev
    real(DP), intent(in ) :: a_Press  (1:NLev)
    real(DP), intent(in ) :: a_Array  (1:NLev)
    integer , intent(in ) :: kmax
    real(DP), intent(in ) :: z_Press(1:kmax)
    real(DP), intent(out) :: z_Array(1:kmax)


    !
    ! local variables
    !
    integer :: k
    integer :: kk


    do k = 1, kmax

      if( z_Press(k) <= a_Press(NLev) ) then
        z_Array(k) = a_Array(NLev)
      else if( z_Press(k) >= a_Press(1) ) then
        z_Array(k) = a_Array(1)
      else
        search_loop : do kk = 2, NLev
          if( a_Press( kk ) < z_Press(k) ) exit search_loop
        end do search_loop
        if( kk > NLev ) &
          stop 'Unexpected error in setting temperature profile'
        z_Array(k) =                                     &
          &   ( a_Array( kk ) - a_Array( kk-1 ) )        &
          & / ( log( a_Press(kk  ) / a_Press( kk-1 ) ) ) &
          & * ( log( z_Press(k   ) / a_Press( kk-1 ) ) ) &
          & + a_Array( kk-1 )
      end if

    end do


  end subroutine Interpolate1D

  !----------------------------------------------------------------------------

end program main
