subroutine SetFlag( &
  & NPress, NTemp,   & ! (in)
  & p_Press, t_Temp, & ! (in)
  & pt_Flag          & ! (out)
  & )

  use vtype_module

  implicit none

  integer , intent(in ) :: NPress
  integer , intent(in ) :: NTemp
  real(dp), intent(in ) :: p_Press(NPress)
  real(dp), intent(in ) :: t_Temp (NTemp )
  logical , intent(out) :: pt_Flag(NPress,NTemp)


  ! Local variables
  !
  integer :: NumTrue
  integer :: NumFalse

  integer :: iPress
  integer :: iTemp


  pt_Flag = .true.
!!$  pt_Flag = .false.

!!$  do iPress = 1, NPress
!!$    do iTemp = 1, NTemp
!!$      if ( iPress == NPress ) then
!!$        pt_Flag(iPress,iTemp) = .false.
!!$      end if
!!$    end do
!!$  end do


  ! If flag is true only around a specified profile, a following line is used.
  ! If not, a following line is commented out.
!!$  call SetFlagFromFile


  NumTrue  = 0
  NumFalse = 0
  do iTemp = 1, NTemp
    do iPress = 1, NPress
      if ( pt_Flag(iPress,iTemp) ) then
        NumTrue  = NumTrue  + 1
      else
        NumFalse = NumFalse + 1
      end if
    end do
  end do
  write( 6, * ) "Number of true     : ", NumTrue
  write( 6, * ) "Number of false    : ", NumFalse
  write( 6, * ) "Total              : ", NumTrue+NumFalse
  write( 6, * ) "Ratio (true/total) : ", real(NumTrue)/real(NumTrue+NumFalse)


  !----------------------------------------------------------------------------

contains

  !----------------------------------------------------------------------------

  subroutine SetFlagFromFile

    use io_profile, only : &
      & IOProfileInqDimLen, &
      & IOProfileRead


    ! Local variables
    !
    character(extstr*2) :: InProfNcFn

    integer               :: kmax
    integer               :: NMol
    integer , allocatable :: m_MolNum(:)
    real(dp), allocatable :: r_Press (:)
    real(dp), allocatable :: r_Temp  (:)
    real(dp), allocatable :: rm_VMR  (:,:)
    real(dp)              :: SurfTemp

    real(dp), allocatable :: z_Press(:)
    real(dp), allocatable :: z_Temp (:)

    integer :: NumTrue
    integer :: NumFalse

    integer :: iPress
    integer :: iTemp
    integer :: i
    integer :: j
    integer :: k


    InProfNcFn = ""
    InProfNcFn = "../prog01.1_mkprofile_Venus/out/Venus_VIRA+GasVMR+CldMMR+UVabsMMR_H2O_CO2_CO_SO2_HF_OCS_N2.nc"

    if ( len_trim( InProfNcFn ) > 0 ) then

      call IOProfileInqDimLen(      &
        & InProfNcFN, "Press", &
        & kmax &
        & )
      kmax = kmax - 1
      call IOProfileInqDimLen(      &
        & InProfNcFN, "MolNum", &
        & NMol &
        & )
      allocate( m_MolNum   (1:NMol) )
      allocate( r_Press    (0:kmax) )
      allocate( r_Temp     (0:kmax) )
      allocate( rm_VMR     (0:kmax,NMol) )
      call IOProfileRead(      &
        & InProfNcFN, "Press", "MolNum", "Temp", "Ts", "VMR", &
        & kmax+1, NMol, &
        & r_Press, m_MolNum, r_Temp, SurfTemp, rm_VMR    &
        & )

      pt_Flag = .false.


      ! check r-level
      do k = 0, kmax
        do iPress = 1, NPress
          if ( p_Press(iPress) <= r_Press(k) ) then
            exit
          end if
        end do
        if ( ( iPress <= 1 ) .or. ( NPress < iPress ) ) then
          write( 6, * ) "ERROR : Pressure range of the table is not enough."
          write( 6, * ) "  ", p_Press(1), r_Press(k), p_Press(NPress)
          stop
        end if
        do iTemp = 1, NTemp
          if ( t_Temp(iTemp) >= r_Temp(k) ) then
            exit
          end if
        end do
        if ( ( iTemp <= 1 ) .or. ( NTemp < iTemp ) ) then
          write( 6, * ) "ERROR : Temperature range of the table is not enough."
          write( 6, * ) "  ", t_Temp(1), r_Temp(k), t_Temp(NTemp)
          stop
        end if
        do j = -1, 0
          do i = -1, 0
            pt_Flag(iPress+i,iTemp+j) = .true.
          end do
        end do
      end do

      ! check z-level
      allocate( z_Press    (1:kmax) )
      allocate( z_Temp     (1:kmax) )
      do k = 1, kmax
        z_Press(k) = ( r_Press(k-1) + r_Press(k) ) / 2.0d0
        z_Temp (k) = ( r_Temp (k-1) + r_Temp (k) ) / 2.0d0
      end do
      do k = 1, kmax
        do iPress = 1, NPress
          if ( p_Press(iPress) <= z_Press(k) ) then
            exit
          end if
        end do
        if ( ( iPress <= 1 ) .or. ( NPress < iPress ) ) then
          write( 6, * ) "ERROR : Pressure range of the table is not enough."
          write( 6, * ) "  ", p_Press(1), z_Press(k), p_Press(NPress)
          stop
        end if
        do iTemp = 1, NTemp
          if ( t_Temp(iTemp) >= z_Temp(k) ) then
            exit
          end if
        end do
        if ( ( iTemp <= 1 ) .or. ( NTemp < iTemp ) ) then
          write( 6, * ) "ERROR : Temperature range of the table is not enough."
          write( 6, * ) "  ", t_Temp(1), z_Temp(k), t_Temp(NTemp)
          stop
        end if
        do j = -1, 0
          do i = -1, 0
            pt_Flag(iPress+i,iTemp+j) = .true.
          end do
        end do
      end do
      deallocate( z_Press )
      deallocate( z_Temp  )

      deallocate( m_MolNum )
      deallocate( r_Press  )
      deallocate( r_Temp   )
      deallocate( rm_VMR   )
    end if

  end subroutine SetFlagFromFile

  !----------------------------------------------------------------------------

end subroutine SetFlag
